#ifndef IMAGESELECTIONLIST_H
#define IMAGESELECTIONLIST_H

#include <QStringList>
#include <QThread>

#include "MDragWidget.h"
#include "WPNetworkEngine.h"
#include "wpcustomviewbasicdata.h"

class QFileInfo;
class ImageSearchThread;
class WPScrollArea;
class CheckableListItem;
class ImageThumbnailThread;

class ImageSelectionList : public MDragWidget, public WPCustomViewBasicData
{
    Q_OBJECT

public:
    ImageSelectionList(const QStringList& aInitialSelections, QWidget* aParent = 0);
    ~ImageSelectionList();
    
    bool MDragWidgetIsDragging() const;
    QSize VisibleSize() const;
    void SearchImages();
    
signals:
    void ImagesSelected(QStringList aItems);

    // Wait note related signals
    void TaskStarted(TWPNetworkEngineCommand aCommand);
    void TaskFinished(TWPNetworkEngineCommand aCommand);

    void CreateThumbnail(QString aFilename, int aIndex);
    void StartThumbnailCreation();

public slots:
    void ImageFound(const QFileInfo& aFileInfo);
    void ThreadFinished();
    void ItemsSelected();
    void CancelClicked();
    void setFocus();
    void UpdateThumbnails();
    void ThumbnailReady(int aIndex, const QImage& aImage);

protected:
    void timerEvent(QTimerEvent* aEvent);

private:
    QString FormatFileSize(int aFileSize) const;
    void keyReleaseEvent(QKeyEvent* event);

private:
    const QStringList iInitialSelections;
    WPScrollArea* iScrollArea;
    
    QList<CheckableListItem*> iItems;
    QList<const QFileInfo*> iFiles;
    ImageSearchThread* iSearchThread;
    ImageThumbnailThread* iThumbnailThread;
};

#endif // IMAGESELECTIONLIST_H
