
#include <QImageReader>

#include "ImageThumbnailThread.h"

ImageThumbnailThread::ImageThumbnailThread(const QSize& aSize, QObject* parent)
    : QThread(parent), iSize(aSize), iThreadCancelled(false)
{
}

ImageThumbnailThread::~ImageThumbnailThread()
{
}

void ImageThumbnailThread::run()
{
    iMutex.lock();
    bool workToDo = !iThreadCancelled && iImageList.size() > 0;
    iMutex.unlock();
    
    while (workToDo)
    {
        QImageReader reader;
        iMutex.lock();
        reader.setFileName(iImageList[0].first);
        iMutex.unlock();
        QSize imageSize = reader.size();

        imageSize.scale(iSize, Qt::KeepAspectRatio);
        reader.setScaledSize(imageSize);
        QImage image(reader.read());
        iMutex.lock();
        emit ThumbnailCreated(iImageList[0].second, image);
        iImageList.removeAt(0);
        workToDo = !iThreadCancelled && iImageList.size() > 0;
        iMutex.unlock();
    }
}

void ImageThumbnailThread::Cancel()
{
    iThreadCancelled = true;
}

void ImageThumbnailThread::AddImage(QString aFilename, int aIndex)
{
    iMutex.lock();
    iImageList.append(QPair<QString, int>(aFilename, aIndex));
    iMutex.unlock();
}
