/*
 * SinglePostView.cpp
 *
 *  Created on: 14.12.2009
 *      Author: apoi
 */

#include <QList>
#include <QVBoxLayout>
#include <QAction>
#include <QString>
#include <QDesktopWidget>

#include "AddComment.h"
#include "BlogView.h"
#include "SinglePostData.h"
#include "SinglePostView.h"
#include "WPData.h"
#include "wpcommentlist.h"
#include "wpscrollarea.h"
#include "WPNetworkEngine.h"
#include "WPData.h"
#include "wpconfirmdialog.h"

//const char* KSinglePostStyleSheet = "background-color: #ECECEC; font-size: 6pt; margin: 5px 0px 3px 0px; color: #333333;";

SinglePostView::SinglePostView(BlogView& aParent) : MDragWidget(&aParent), iParent(aParent)
{
    setWindowFlags(Qt::Window);
#ifdef Q_WS_MAEMO_5
    setAttribute(Qt::WA_Maemo5StackedWindow);
#endif
    setWindowTitle(tr("Single post view"));
    SetScreenMaxSize();

    iListbox = NULL;
    QVBoxLayout* layout = new QVBoxLayout(this);

    //setStyleSheet(KSinglePostStyleSheet);
    
    iScrollArea = new WPScrollArea(this);
    layout->setSpacing(0);
    layout->setContentsMargins(0, 0, 0, 0);
    layout->addWidget(iScrollArea);
    iScrollArea->setWidgetResizable(true);
    iScrollArea->setFocusPolicy(Qt::NoFocus);
    iScrollArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);

    // Prepare softkeys
    //
    QAction* rightSoftKey = new QAction(tr("Back"), this);
    connect(rightSoftKey, SIGNAL(triggered()), this, SLOT(CloseView()));
    rightSoftKey->setSoftKeyRole(QAction::NegativeSoftKey);

    // Finally add the data for WPCustomViewBasicData
    //
    iTitleText = tr("Single post view");
    iTabText = tr("Post");
    iHappyAction = NULL;
    iSadAction = rightSoftKey;
}

SinglePostView::~SinglePostView()
{

}

void SinglePostView::SetScreenMaxSize()
{
    setMaximumWidth(QApplication::desktop()->screenGeometry().size().width());
}

void SinglePostView::keyReleaseEvent(QKeyEvent* event)
{
    qDebug("WP::SinglePostView::keyPressEvent Key event %d", event->key());
    if (event->key() == Qt::Key_Context1 && iHappyAction)
    {
        iHappyAction->trigger();
        event->accept();
    }
    else if (event->key() == Qt::Key_Context2 && iSadAction)
    {
        iSadAction->trigger();
        event->accept();
    }
    else
    {
        event->ignore();
    }
}

void SinglePostView::resizeEvent(QResizeEvent *e)
{
    SetScreenMaxSize();
    QWidget::resizeEvent(e);
}

void SinglePostView::ShowPost(const WPDataBlog& aBlog, WPPost aPost)
{
    iPost = aPost;

    // forget old list
    // XXX: this is wrong, iScrollArea has ownership of iListbox?
    iListbox->deleteLater();
    iListbox = NULL;

    // find comments
    QList<WPComment> comments = aPost->GetComments();

    // display
    // These are deleted when the layout (in iListBox) gets deleted
    iPostData = new SinglePostData(this, aPost);
    iListbox = new WPCommentList(comments, iParent.GetGravatars(), *iScrollArea, iParent.GetThemeData(), true);
    iScrollArea->setWidget(iPostData);
    iScrollArea->addWidget(iListbox);
    iScrollArea->AddStretch();

    // signals
    connect(
       &aBlog,  SIGNAL(CommentAdded(WPComment)), 
        iListbox, SLOT(CommentAdded(WPComment))
    );
    connect(
       &aBlog,  SIGNAL(CommentChanged(WPComment)), 
        iListbox, SLOT(CommentChanged(WPComment))
    );
    connect(
       &aBlog,  SIGNAL(CommentRemoved(WPComment)), 
        iListbox, SLOT(CommentRemoved(WPComment))
    );

    if (!aPost->IsLocal()) {
        iAddComment = new AddComment(this, aPost, iParent);
        iScrollArea->addWidget(iAddComment);
        QAction* lsk = new QAction(tr("Publish"), this);
        connect(lsk, SIGNAL(triggered()), iAddComment, SLOT(SaveComment()));
        iHappyAction = lsk;
    }

    iListbox->setFocus();
    ScrollToTop();
}

void SinglePostView::UpdateData(const WPDataBlog& aBlog)
{
    if (iPost) {
        ShowPost(aBlog, iPost);
    }
}

bool SinglePostView::MDragWidgetIsDragging() const
{
    return iScrollArea->IsDragging();
}

QSize SinglePostView::VisibleSize() const
{
    return iScrollArea->GetVisibleContentSize();
}

void SinglePostView::ScrollToTop()
{
    iScrollArea->ScrollToTop();
}

void SinglePostView::CloseView()
{
    if(iAddComment && iAddComment->HasText())
    {
        WP_CONFRIM_AND_RETURN_ON_FALSE;
    }

    iPost = NULL;
    iParent.ShowTabbedListView();
}
