/*
 * ExplanatoryLineEdit.cpp
 *
 *  Created on: 11.1.2010
 *      Author: apoi
 */

#include "explanatorylineedit.h"

#include <iostream>

ExplanatoryLineEdit::ExplanatoryLineEdit(QWidget *aParent, const QString& aExplanation)
    : QLineEdit(aParent), iExplanation(aExplanation)
{
    connect(this, SIGNAL(textEdited(const QString&)), this, SLOT(textEdited(const QString&)));
    iMaskPassword = false;
    setText(QString());
}

ExplanatoryLineEdit::~ExplanatoryLineEdit()
{
}

void ExplanatoryLineEdit::SetPasswordMasking(bool aMaskPassword)
{
    iMaskPassword = aMaskPassword;
}

void ExplanatoryLineEdit::setText(const QString& aValue)
{
    iValue = aValue;
    if (!aValue.isEmpty())
        SetMaskedText(iValue);
    else
        QLineEdit::setText(iExplanation);
}

QString ExplanatoryLineEdit::text() const
{
    return iValue;
}

void ExplanatoryLineEdit::SetExplanation(const QString& aExplanation)
{
    iExplanation = aExplanation;
    if (iValue.isEmpty())
        QLineEdit::setText(iExplanation);
}

void ExplanatoryLineEdit::focusInEvent(QFocusEvent *e)
{
    QLineEdit::setText(iValue);
    QLineEdit::focusInEvent(e);
}

void ExplanatoryLineEdit::focusOutEvent(QFocusEvent *e)
{
    setText(QLineEdit::text());
    QLineEdit::focusOutEvent(e);
}

void ExplanatoryLineEdit::hideEvent(QHideEvent *)
{
    // Maemo workaround
    clearFocus();
}

void ExplanatoryLineEdit::textEdited(const QString& aText)
{
    iValue = aText;
}

void ExplanatoryLineEdit::SetMaskedText(const QString& aText)
{
    if (iMaskPassword && !hasFocus())
    {
        QString masked(aText);
        QLineEdit::setText(masked.fill('*'));
    }
    else
    {
        QLineEdit::setText(aText);
    }
}

void ExplanatoryLineEdit::Clear()
{
    QLineEdit::clear();
    iValue.clear();
}
