#include <QVBoxLayout>
#include <QFont>
#include <QEvent>
#include <QKeyEvent>

#include "textlistitem.h"
#include "clippinglabel.h"

TextListItem::TextListItem(const QString& aItemId, const QString& aTitle,
                           const QString& aFrom, const QStringList& aDetails, QWidget *parent)
                               : QFrame(parent)
{
    iItemId = aItemId;
    iKeyPressReceived = false;
    setContextMenuPolicy(Qt::NoContextMenu);
    setFocusPolicy(Qt::StrongFocus);

    setFrameShape(QFrame::StyledPanel);
    setFrameShadow(QFrame::Raised);

    QVBoxLayout* layout = new QVBoxLayout(this);
    layout->setSpacing(0);
    layout->setContentsMargins(0, 0, 0, 0);

    AddTitleLabel(aTitle);
    if (!aFrom.isEmpty())
        AddFromLabel(aFrom);
    for (int i = 0; i < aDetails.count(); i++)
        AddDetailLabel(aDetails[i]);
}

TextListItem::~TextListItem()
{

}

void TextListItem::AddTitleLabel(const QString& aTitleText)
{
    ClippingLabel *titleLabel = new ClippingLabel(this);
    titleLabel->setObjectName(QString::fromLatin1("titleLabel"));
    titleLabel->setIndent(8);
    titleLabel->setText(aTitleText);
    QSizePolicy sizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Maximum);
    titleLabel->setSizePolicy(sizePolicy);
    layout()->addWidget(titleLabel);
}

void TextListItem::AddDetailLabel(const QString& aDetailText)
{
    ClippingLabel *label = new ClippingLabel(this);
    label->setObjectName(QString::fromLatin1("detailLabel"));
    label->setIndent(8);
    label->setText(aDetailText);
    QSizePolicy sizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Maximum);
    label->setSizePolicy(sizePolicy);
    layout()->addWidget(label);
}

void TextListItem::AddFromLabel(const QString& aDetailText)
{
    ClippingLabel *label = new ClippingLabel(this);
    label->setObjectName(QString::fromLatin1("fromLabel"));
    label->setIndent(8);
    QString fromFieldText;
    fromFieldText = tr("by");
    fromFieldText += ' ';
    fromFieldText += aDetailText;
    label->setText(fromFieldText);
    QSizePolicy sizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Maximum);
    label->setSizePolicy(sizePolicy);
    layout()->addWidget(label);
}

void TextListItem::mouseReleaseEvent(QMouseEvent *e)
{
    if (hasFocus())
        emit TextListItemSelected(this);
    QFrame::mouseReleaseEvent(e);
}

void TextListItem::keyPressEvent(QKeyEvent *e)
{
    if (e->key() == Qt::Key_Select || (e->key() == Qt::Key_Enter))
    {
        // Check later if we got the full click, or just the release event.
        // If an user clicks a softkey menu, the press is consumed but
        // release is not -- we do not want partial event to trigger the
        // popup menu.
        iKeyPressReceived = true;
    }
    else
    {
        QFrame::keyPressEvent(e);
    }
}

void TextListItem::keyReleaseEvent(QKeyEvent *e)
{
    if ((e->key() == Qt::Key_Select || e->key() == Qt::Key_Enter) && iKeyPressReceived)
    {
        iKeyPressReceived = false;
        emit TextListItemSelected(this);
    }
    else
    {
        QFrame::keyReleaseEvent(e);
    }
}

QPoint TextListItem::TopRight() const
{
    return geometry().topRight();
}
