/*
 * themedatastorage.cpp
 *
 *  Created on: 27.5.2010
 *      Author: jlep
 */

#include <QFont>
#include <QPixmap>
#include <QLinearGradient>

#include "wpcommongeometry.h"
#include "themedatastorage.h"
#include "utils.h"

ThemeData::ThemeData() :
      iTabGradientBrush(NULL),
      iTabFont(NULL),
      iListItemPixmap(NULL),
      iListTitleFont(NULL),
      iListContentFont(NULL),
      iListInfoFont(NULL),
      iListDetailFont(NULL)
{
    iIcons.clear();
}

ThemeData::~ThemeData()
{
    delete iTabGradientBrush;
    delete iTabFont;
    delete iListItemPixmap;
    
    delete iListTitleFont;
    delete iListContentFont;
    delete iListInfoFont;
    delete iListDetailFont;

    iIcons.clear();
}

void ThemeData::Init()
{
    // Tabs
    iTabBarHeight = Utils::smallScreen() ? 27 : 60;
    iMinTabSize = Utils::smallScreen() ? QSizeF(20, 10) :
                                         QSizeF(80, 55); 
    QLinearGradient ql(0, 0, 0, iTabBarHeight);
    ql.setColorAt(0.0, QColor(238,238,238));
    ql.setColorAt(0.5, QColor(216,216,216));
    ql.setColorAt(1.0, QColor(203,203,203));
    iTabGradientBrush = new QBrush(ql);
    iTabFont = new QFont("Arial");
    iTabFont->setPixelSize(Utils::smallScreen() ? 12 : 10);    
    const int iconCount = 12;
    QPixmap iconSheet(":/images/icons32-vs.png");
    const int iconWidth = iconSheet.width()/iconCount; 
    
    for (int i=0; i < 12; i++)
    {
        iIcons.append(iconSheet.copy(i*iconWidth,0,iconWidth,iconSheet.height()));
    }

    iContextMenuIcons.clear();
    iContextMenuIcons.append(QPixmap(":/images/approve.png"));
    iContextMenuIcons.append(QPixmap(":/images/edit.png"));
    iContextMenuIcons.append(QPixmap(":/images/open.png"));
    iContextMenuIcons.append(QPixmap(":/images/remove.png"));
    iContextMenuIcons.append(QPixmap(":/images/reply.png"));
    iContextMenuIcons.append(QPixmap(":/images/spam.png"));
    iContextMenuIcons.append(QPixmap(":/images/view.png"));
    
    // Lists
    iListItemPixmap = new QPixmap(":/images/listitembg.png");
    iListItemShadowMargin = 4;
    
    iListTitleFont = new QFont("Arial");
    iListTitleFont->setPixelSize(Utils::smallScreen() ? 12 : 20);
	
    iListContentFont = new QFont("Arial");
    iListContentFont->setPixelSize(Utils::smallScreen() ? 12 : 14);
	
	iListInfoFont = new QFont("Arial");
	iListInfoFont->setPixelSize(Utils::smallScreen() ? 12 : 14);
	iListInfoFont->setItalic(true);
	
	iListDetailFont = new QFont("Arial");
	iListDetailFont->setPixelSize(Utils::smallScreen() ? 10 : 11);
	
	iListItemPadding = Utils::smallScreen() ? 5 : 10;
	iListShowCommentIcons = !Utils::smallScreen();
	
    #ifdef Q_WS_MAEMO_5
	iListTitleHeight = 45;
    #else
	iListTitleHeight = Utils::smallScreen() ? 20 : 32;
    #endif 
	
	iListItemSpacing = Utils::smallScreen() ? 3 : 5;
}
