#include "wpbloggraphicslistitem.h"
#include "WPData.h"

WPBlogGraphicsListItem::WPBlogGraphicsListItem(const WPDataBlog& blog, BusyIndicator& busyIndicator, ThemeData& themeData)
  : WPGraphicsListItem(busyIndicator, themeData)
  , iBlog(blog)
  , iTitle(blog.iBlogName)
{
	int draftPosts = 0;
	int unapprovedComments = 0;
	QDateTime latestActivity;
	QDateTime comparisonDate;

	for (QListIterator<WPPost> i = blog.GetPosts(); i.hasNext(); ) {
		const WPPost post = i.next();

		if (post->GetLocalTime() > latestActivity)
			latestActivity = post->GetLocalTime();

		if (post->IsDraft())
			draftPosts++;
	}

	for (QListIterator<WPComment> j = blog.GetComments(); j.hasNext(); ) {
		WPComment comment = j.next();

		if (comment->GetLocalTime() > latestActivity)
			latestActivity = comment->GetLocalTime();

		if (comment->IsUnapproved())
			unapprovedComments++;
	}

	if (unapprovedComments == 1)
		iStatus += tr("1 unapproved comment");
	else if (unapprovedComments > 1)
		iStatus += QString::number(unapprovedComments) + tr(" unapproved comments");

	if (draftPosts > 0)
	{
		if (!iStatus.isEmpty())
			iStatus += ", ";
		if (draftPosts == 1)
			iStatus += tr("1 draft");
		else
			iStatus += QString::number(draftPosts) + tr(" drafts");
	}

	iActivity = tr("Latest activity: ") + WPNetworkEngine::LocalizedTime(latestActivity);;
	
	setMinimumHeight(ItemMinimumHeight);
}

WPBlogGraphicsListItem::~WPBlogGraphicsListItem()
{

}

void WPBlogGraphicsListItem::paintContent(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget)
{
	painter->setPen(Qt::black);

	painter->setFont(*iThemeData.iListTitleFont);
	painter->drawText(QPoint(contentXOffset(), 24), iTitle);
	painter->setFont(*iThemeData.iListInfoFont);
	painter->drawText(QPoint(contentXOffset(), 56), iStatus);
	painter->drawText(QPoint(contentXOffset(), 88), iActivity);
}

bool WPBlogGraphicsListItem::IsBusy()
{
	return false;
}

bool WPBlogGraphicsListItem::HasError()
{
	return false;
}

void WPBlogGraphicsListItem::handleGeomeryChange()
{
	calculateElidedTexts();
}

const WPDataBlog& WPBlogGraphicsListItem::getBlog() const
{
	return iBlog;
}

void WPBlogGraphicsListItem::calculateElidedTexts()
{
	QFontMetrics titleFm(*iThemeData.iListTitleFont);
	iTitle = titleFm.elidedText(iBlog.iBlogName, Qt::ElideRight,
			                    int(width()) - (2 * contentXOffset()));
}
