#include <QGraphicsLinearLayout>
#include <QGraphicsProxyWidget>
#include <QLabel>

#include "wpcommentgraphicslistitem.h"
#include "wpcommentgraphicslist.h"
#include "WPData.h"

#include "PopupMenu.h"
#include "themedatastorage.h"
#include "BlogView.h"
#include "wpcommongeometry.h"
#include "wpconfirmdialog.h"

WPCommentGraphicsList::WPCommentGraphicsList(
        BlogView* aBlogView, const QGraphicsView& aGraphicsView,
        PannableView* aPannableView
    ) : WPGraphicsList(aBlogView, aGraphicsView, aPannableView),
        iGravatars(aBlogView->GetGravatars())
{
    // TODO find a proper way to align menu icon and text side by side.
    iActionApprove = new QAction(iThemeData.iContextMenuIcons[ThemeData::EApprove],tr("    Approve"), this);
    iActionUnapprove = new QAction(iThemeData.iContextMenuIcons[ThemeData::EApprove],tr("    Unapprove"), this);
    iActionView = new QAction(iThemeData.iContextMenuIcons[ThemeData::EView],tr("    View"), this);
    iActionHide = new QAction(iThemeData.iContextMenuIcons[ThemeData::EView],tr("    Minimize"), this);
    iActionReply = new QAction(iThemeData.iContextMenuIcons[ThemeData::EReply],tr("    Reply"), this);
    iActionDelete = new QAction(iThemeData.iContextMenuIcons[ThemeData::ERemove],tr("    Delete"), this);
    iActionSpam = new QAction(iThemeData.iContextMenuIcons[ThemeData::ESpam],tr("    Spam"), this);

    connect(iActionApprove,   SIGNAL(triggered()), this, SLOT(Approve()));
    connect(iActionUnapprove, SIGNAL(triggered()), this, SLOT(Unapprove()));
    connect(iActionView,      SIGNAL(triggered()), this, SLOT(View()));
    connect(iActionHide,      SIGNAL(triggered()), this, SLOT(View()));
    connect(iActionReply,     SIGNAL(triggered()), this, SLOT(Reply()));
    connect(iActionDelete,    SIGNAL(triggered()), this, SLOT(Delete()));
    connect(iActionSpam,      SIGNAL(triggered()), this, SLOT(Spam()));

    connect(
        iGravatars, SIGNAL(GravatarUpdated (const QString &, const QPixmap &)),
        this,         SLOT(GravatarUpdated (const QString &, const QPixmap &))
    );    
    
    connect(this,      SIGNAL(ReplyToComment(WPComment)),
            aBlogView, SLOT(ReplyToComment(WPComment)));
    
    iUnapprovedLabel = new QLabel;
    iUnapprovedProxy = new QGraphicsProxyWidget(this);
    iUnapprovedProxy->setWidget(iUnapprovedLabel);
    iUnapprovedLabel->setObjectName(QString::fromLatin1("unapprovedLabel"));
    iUnapprovedLabel->setFixedHeight(iThemeData.iListTitleHeight);

    iApprovedLabel = new QLabel;
    iApprovedProxy = new QGraphicsProxyWidget(this);
    iApprovedProxy->setWidget(iApprovedLabel);
    iApprovedLabel->setObjectName(QString::fromLatin1("approvedLabel"));
    iApprovedLabel->setFixedHeight(iThemeData.iListTitleHeight);
    
    setContentsMargins(0, 0, 0, 0);
    iLayout->addItem(iUnapprovedProxy);
    iUnapprovedLayout = AddLayout();
    iLayout->addItem(iApprovedProxy);
    iApprovedLayout = AddLayout();
    
    // update label text, and container height
    UpdateLabels();
}

WPCommentGraphicsList::~WPCommentGraphicsList()
{

}

void WPCommentGraphicsList::AddComment(WPComment aComment, bool aShowFullComment)
{
    WPCommentGraphicsListItem* item = new WPCommentGraphicsListItem(aComment, aShowFullComment, iBusyIndicator, iThemeData);
    connect(item, SIGNAL(Selected(WPGraphicsListItem*)), this, SLOT(ShowContextMenu(WPGraphicsListItem*)));

    // get gravatar for item
    item->SetAvatarImage(iGravatars->GetGravatar(aComment));
    
    if(aComment->IsUnapproved())
        {
        OrderedInsert(*iUnapprovedLayout, *item);
        }
    else
        {
        OrderedInsert(*iApprovedLayout, *item);
        }
    
    UpdateLabels();
}

void WPCommentGraphicsList::RemoveComment (WPComment aComment)
{
    WPCommentGraphicsListItem * item = 0;
    
    item = RemoveFromLayout(*iUnapprovedLayout, aComment);
    if(!item) { item = RemoveFromLayout(*iApprovedLayout, aComment); }
    
    delete item;
    UpdateLabels();
}

void WPCommentGraphicsList::GravatarUpdated (const QString &email, const QPixmap &pixmap)
{
    UpdateGravatarsInLayout(*iUnapprovedLayout, email, pixmap);
    UpdateGravatarsInLayout(*iApprovedLayout, email, pixmap);
}

void WPCommentGraphicsList::PopulatePopupMenuForCurrentItem(PopupMenu* aPopupMenu)
{
	if (CurrentItem()->GetComment()->IsUnapproved())
	{
		aPopupMenu->addAction(iActionApprove);
		aPopupMenu->setActiveAction(iActionApprove);
	}
	else
	{
		aPopupMenu->addAction(iActionUnapprove);
		aPopupMenu->setActiveAction(iActionUnapprove);
	}
	if (CurrentItem()->ShowFullComment())
		aPopupMenu->addAction(iActionHide);
	else
		aPopupMenu->addAction(iActionView);

	// XXX: if (iNetworkEngine.GetPost(aItem->GetComment().iPostId))
		aPopupMenu->addAction(iActionReply);

	aPopupMenu->addAction(iActionDelete);
	if (CurrentItem()->GetComment()->iStatus != QLatin1String("spam"))
		aPopupMenu->addAction(iActionSpam);
}

void WPCommentGraphicsList::Approve()
{
    CurrentItem()->GetComment()->UpdateStatus("approve");
}

void WPCommentGraphicsList::Unapprove()
{
    CurrentItem()->GetComment()->UpdateStatus("hold");
}

void WPCommentGraphicsList::View()
{
    CurrentItem()->ChangeViewMode();
}

void WPCommentGraphicsList::Reply()
{
    emit ReplyToComment(CurrentItem()->GetComment());
}

void WPCommentGraphicsList::Delete()
{
    WP_CONFRIM_AND_RETURN_ON_FALSE;

    CurrentItem()->GetComment()->Delete();
}

void WPCommentGraphicsList::Spam()
{
   CurrentItem()->GetComment()->UpdateStatus("spam");
}

void WPCommentGraphicsList::UpdateCommentsData(const WPDataBlog& aBlog)
{
    if(iCurrentBlog) {
        disconnect(iCurrentBlog, NULL, this, NULL);
    }
    iCurrentBlog = const_cast<WPDataBlog *>(&aBlog);
    
    ClearLayout(*iUnapprovedLayout);
    ClearLayout(*iApprovedLayout);
    
    // add comments, inserting labels as needed
    QListIterator<WPComment> data(aBlog.GetComments());
    while (data.hasNext()) {
        WPComment comment = data.next();

        // sorted insert, with gravatar
        AddComment(comment, false);
    }
    
    connect(
       &aBlog,  SIGNAL(CommentAdded(WPComment)), 
        this,   SLOT(CommentAdded(WPComment))
    );
    connect(
       &aBlog,  SIGNAL(CommentChanged(WPComment)), 
        this,   SLOT(CommentChanged(WPComment))
    );
    connect(
       &aBlog,  SIGNAL(CommentRemoved(WPComment)), 
        this,   SLOT(CommentRemoved(WPComment))
    );
    
    UpdateLabels();
    setFocus();
}

void WPCommentGraphicsList::CommentChanged(WPComment aComment)
{
    qDebug("WPCommentGraphicsList::CommentChanged");

    // cycle
    RemoveComment(aComment);
    AddComment(aComment, false);
    
    UpdateLabels();
}

void WPCommentGraphicsList::CommentRemoved(WPComment aComment)
{
    qDebug("WPCommentGraphicsList::CommentRemoved");

    RemoveComment(aComment);

    UpdateLabels();
}

void WPCommentGraphicsList::CommentAdded(WPComment aComment)
{
    qDebug("WPCommentGraphicsList::CommentAdded");

    AddComment(aComment, false);

    UpdateLabels();
}

void WPCommentGraphicsList::UpdateLabels()
{
    int unapproved = iUnapprovedLayout->count();
    if (unapproved) {
        iUnapprovedLabel->setText(tr("Unapproved") + " (" + QString::number(unapproved) + ')');
        UncollapseItem(iUnapprovedProxy);
    } else {
        CollapseItem(iUnapprovedProxy);
    }

    int approved = iApprovedLayout->count();
    if (approved) {
        iApprovedLabel->setText(tr("Comments") + " (" + QString::number(approved) + ')');
        UncollapseItem(iApprovedProxy);
    } else {
        CollapseItem(iApprovedProxy);
    }
}

void WPCommentGraphicsList::OrderedInsert(QGraphicsLinearLayout & layout, WPCommentGraphicsListItem & item)
{
    int i;
    for(i = 0; i < layout.count(); ++i) {
        WPCommentGraphicsListItem * cListItem = LayoutToListItem(layout.itemAt(i));
        if(*cListItem < item) { break; }
    }
    layout.insertItem(i, &item);
}

WPCommentGraphicsListItem * WPCommentGraphicsList::RemoveFromLayout(QGraphicsLinearLayout & layout, WPComment comment)
{
    for(int i = 0; i < layout.count(); ++i) {
        WPCommentGraphicsListItem * item = LayoutToListItem(layout.itemAt(i));
        if(item->GetComment() == comment) {
            IteratorGuard itGuard(*this, item);
            layout.removeAt(i);
            return item;
        }
    }
    return 0;
}

void WPCommentGraphicsList::UpdateGravatarsInLayout(QGraphicsLinearLayout & layout, const QString &email, const QPixmap &pixmap)
{
    // look for ALL items with matching email and set their pixmap
    for (int i = 0; i < layout.count(); i++) {
        WPCommentGraphicsListItem *item = LayoutToListItem(layout.itemAt(i));

        // normalize for comparison
        QString normalizedEmail = WPDataGravatars::NormalizeEmail(item->GetComment()->iAuthorEmail);

        if (normalizedEmail == email)
            // update item's pixmap
            item->SetAvatarImage(pixmap);
    }
}
