
#include <QBoxLayout>
#include <QEvent>
#include <QPoint>
#include <QKeyEvent>
#include <QRect>
#include <QDesktopWidget>

#include "WPData.h"
#include "wpcommentlistitem.h"
#include "WPNetworkEngine.h"
#include "MDragWidget.h"

static const int KCommentDefaultHeight = 57;

WPCommentListItem::WPCommentListItem(MDragWidget& aParent, WPComment aComment, bool aShowFullComment)
    : QFrame(&aParent), iParent(aParent), iComment(aComment)
{
    ui.setupUi(this);

    iShowFullComment = aShowFullComment;
    iKeyPressReceived = false;

    ui.postTitle->setText(aComment->iPostTitle);
    ui.postTitle->setTextFormat(Qt::RichText);
    ui.authorName->setText(aComment->iAuthor);
    ui.timeStamp->setText(aComment->GetShortLocalTimeString());
    ui.commentText->setText(aComment->iContent);
    ui.url->setText(aComment->iAuthorUrl);
    ui.email->setText(aComment->iAuthorEmail);

    SetViewMode(iShowFullComment);
    setFrameStyle(QFrame::Box);
    setContextMenuPolicy(Qt::NoContextMenu);
}

WPCommentListItem::~WPCommentListItem()
{

}

void WPCommentListItem::SetAvatarImage(const QPixmap& aPixmap)
{
    if (!aPixmap.isNull())
        ui.avatar->setPixmap(aPixmap);
}

void WPCommentListItem::mouseReleaseEvent(QMouseEvent *e)
{
    if (hasFocus() && !iParent.MDragWidgetIsDragging())
    {
        emit CommentItemSelected(this);
    }
    QFrame::mouseReleaseEvent(e);
}

void WPCommentListItem::keyPressEvent(QKeyEvent *e)
{
    if (e->key() == Qt::Key_Select || e->key() == Qt::Key_Enter)
    {
        // Check later if we got the full click, or just the release event.
        // If an user clicks a softkey menu, the press is consumed but
        // release is not -- we do not want partial event to trigger the
        // popup menu.
        iKeyPressReceived = true;
    }
    else
    {
        QFrame::keyPressEvent(e);
    }
}

void WPCommentListItem::keyReleaseEvent(QKeyEvent *e)
{
    if((e->key() == Qt::Key_Select || e->key() == Qt::Key_Enter) && iKeyPressReceived)
    {
        iKeyPressReceived = false;
        emit CommentItemSelected(this);
    }
    else
    {
        QFrame::keyReleaseEvent(e);
    }
}

void WPCommentListItem::resizeEvent(QResizeEvent *e)
{
    // Gravatar = 80
    // 3x3 horizontal spacing within the layout = 9
    // Left edge padding = 8
    // Border on both edges = 2
    const int modifier = 80 + 9 + 8 + 2;
    const int width = iParent.VisibleSize().width() - modifier;

    ui.postTitle->setMaximumWidth(width);
    ui.commentText->setMaximumWidth(width);
    ui.email->setMaximumWidth(width);
    ui.url->setMaximumWidth(width);

    QFrame::resizeEvent(e);
}

WPComment WPCommentListItem::GetComment() const
{
    return iComment;
}

QPoint WPCommentListItem::TopRight() const
{
    return geometry().topRight();
}

void WPCommentListItem::ChangeViewMode()
{
    iShowFullComment = !iShowFullComment;
    SetViewMode(iShowFullComment);
}

void WPCommentListItem::SetViewMode(bool aShowFullComment)
{
    if (aShowFullComment)
    {
        ui.commentText->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
        ui.commentText->setWordWrap(true);
        ui.commentText->setMaximumHeight(QWIDGETSIZE_MAX);
    }
    else
    {
        ui.commentText->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
        ui.commentText->setFixedHeight(KCommentDefaultHeight);
    }
}

void WPCommentListItem::Update (WPComment aComment)
{
    iComment = aComment;
    ui.postTitle->setText(aComment->iPostTitle);
    ui.authorName->setText(aComment->iAuthor);
    ui.timeStamp->setText(aComment->GetShortLocalTimeString());
    ui.commentText->setText(aComment->iContent);
    ui.url->setText(aComment->iAuthorUrl);
    ui.email->setText(aComment->iAuthorEmail);
}
