#include <QPoint>
#include <QRect>
#include <QPainter>

#include "wppagegraphicslistitem.h"
#include "WPData.h"
#include "themedatastorage.h"

WPPageGraphicsListItem::WPPageGraphicsListItem(WPPage aPage, BusyIndicator& aBusyIndicator, ThemeData& aThemeData)
    : WPGraphicsListItem(aBusyIndicator, aThemeData)
    , iPage(aPage)
{
    setObjectName(QString::fromLatin1("WPPageListItem"));
    iDate = aPage->GetShortLocalTimeString(); 
    setMinimumHeight(ItemMinimumHeight);
}

WPPageGraphicsListItem::~WPPageGraphicsListItem()
{
}

WPPage WPPageGraphicsListItem::GetPage() 
{
    return iPage;
}

bool WPPageGraphicsListItem::operator<(WPPageGraphicsListItem const & other)
{
    return (iPage->GetLocalTime() < other.iPage->GetLocalTime());
}

void WPPageGraphicsListItem::paintContent(QPainter* painter, const QStyleOptionGraphicsItem* /*option*/, QWidget* /*widget*/)
{   
    painter->setPen(Qt::black);

    // TODO: Add elided texts
    painter->setFont(*iThemeData.iListTitleFont);
    painter->drawText(QPoint(contentXOffset(), 24), iTitle);
    painter->setFont(*iThemeData.iListInfoFont);
    painter->drawText(QPoint(contentXOffset(), 56), "by " + iPage->iWpAuthorDisplayName);
    painter->setFont(*iThemeData.iListDetailFont);
    painter->drawText(QPoint(contentXOffset(), 112), iDate);
}

bool WPPageGraphicsListItem::IsBusy()
{
    return iPage->IsBusy();
}

bool WPPageGraphicsListItem::HasError()
{
	return iPage->iError;
}

void WPPageGraphicsListItem::handleGeomeryChange()
{
	calculateElidedTexts();
}

void WPPageGraphicsListItem::calculateElidedTexts()
{
    QFontMetrics titleFm(*iThemeData.iListTitleFont);
    iTitle = titleFm.elidedText(iPage->iTitle, Qt::ElideRight, int(width()) - 50);
}

