#include "wppostgraphicslistitem.h"
#include "WPData.h"
#include "themedatastorage.h"

#include <QPoint>
#include <QRect>
#include <QPainter>

WPPostGraphicsListItem::WPPostGraphicsListItem(WPPost aPost, BusyIndicator& aBusyIndicator, ThemeData& aThemeData)
    : WPGraphicsListItem(aBusyIndicator, aThemeData)
    , iPost(aPost)
{
    setObjectName(QString::fromLatin1("WPPostListItem"));
    iDate = aPost->GetShortLocalTimeString();
    int commentCount = iPost->GetComments().count();
    if(commentCount == 1) {
        iCommentsText = tr("1 comment");
    } else {
        iCommentsText = tr("%1 comments") .arg(commentCount);
    }
    setMinimumHeight(ItemMinimumHeight);
}

WPPostGraphicsListItem::~WPPostGraphicsListItem()
{
}

bool WPPostGraphicsListItem::operator<(WPPostGraphicsListItem const & other)
{
    return (iPost->GetLocalTime() <other.iPost->GetLocalTime()); 
}

WPPost WPPostGraphicsListItem::GetPost() const
{
    return iPost;
}

void WPPostGraphicsListItem::paintContent(QPainter* painter, const QStyleOptionGraphicsItem* /*option*/, QWidget* /*widget*/)
{
    painter->setPen(Qt::black);

    // TODO: Add elided texts
    painter->setFont(*iThemeData.iListTitleFont);
    painter->drawText(QPoint(contentXOffset(), 24), iTitle);
    painter->setFont(*iThemeData.iListInfoFont);
    painter->drawText(QPoint(contentXOffset(), 56), "by " + iPost->iWpAuthorDisplayName);
    painter->setFont(*iThemeData.iListDetailFont);
    painter->drawText(QPoint(contentXOffset(), 112), iDate);
    painter->drawText(QPoint(contentXOffset(), 132), iCommentsText);
}

bool WPPostGraphicsListItem::IsBusy()
{
    return iPost->IsBusy();
}

bool WPPostGraphicsListItem::HasError()
{
    return iPost->iError;
}

void WPPostGraphicsListItem::handleGeomeryChange()
{
    calculateElidedTexts();
}

void WPPostGraphicsListItem::calculateElidedTexts()
{
    QFontMetrics titleFm(*iThemeData.iListTitleFont);
    iTitle = titleFm.elidedText(iPost->iTitle, Qt::ElideRight, int(width()) - 50);
}
